Lua_HeroGeniusData = class("Lua_HeroGeniusData")
function Lua_HeroGeniusData:ctor()
	self:superCall("ctor");
	self.heroDic = {}; -- 英雄装备的天赋技能
end

function Lua_HeroGeniusData:dispose()
	self.heroDic = {};
end

-- 天赋配置
function Lua_HeroGeniusData:getGeniusConfig(id)
	return getConfig("hero_talent", id);
end

-- 天赋解锁条件
function Lua_HeroGeniusData:getUnlockCond(level)
    return getConfig("hero_const", "talent_unlock_cond", "arr_value", level);
end

function Lua_HeroGeniusData:updateHeroGenius(msg)
	local uniqueId = tonumber(msg.unique_id);

	self:removeHeroGenius(uniqueId);

	local info = msg.talent_skill;
	if info then
		if self.heroDic[uniqueId] == nil then
			self.heroDic[uniqueId] = {};
		end
		local level;
		local skillId;
		for i = 1, #info do
			level = info[i].key;
			skillId = info[i].value;
			self.heroDic[uniqueId][level] = skillId;
		end
	end
end

function Lua_HeroGeniusData:removeHeroGenius(heroUnId)
	local dic = self.heroDic[heroUnId];
	if dic then
		for k, v in pairs(dic) do
			dic[k] = nil;
		end
	end
end

function Lua_HeroGeniusData:getHeroGeniusSkill(heroUid, level)
	heroUid = tonumber(heroUid);
	if self.heroDic[heroUid] == nil then
		return 0;
	end
	return self.heroDic[heroUid][level] or 0;
end

function Lua_HeroGeniusData:getHeroGeniusList(heroId)
	if self.geniusDic == nil then
		self.geniusDic = {};
		local config = getConfig("hero_talent");
		for k, v in pairs(config) do
			if self.geniusDic[v.hero_id] == nil then
				self.geniusDic[v.hero_id] = Array();
			end
			self.geniusDic[v.hero_id]:add(k);
		end
		for _, v in pairs(self.geniusDic) do
			quickSort(v, nil, true);
		end
	end
	return self.geniusDic[heroId];
end

function Lua_HeroGeniusData:checkExistGenius(heroId)
	return self:getHeroGeniusList(heroId) ~= nil;
end

function Lua_HeroGeniusData:checkHeroGeniusIsUnlock(heroVo, bTips)
    local arr = self:getHeroGeniusList(heroVo.id);
    local config = self:getGeniusConfig(arr[1]);
    local bUnlock = self:checkGeniusLvIsUnlock(config.talent_lvl, heroVo.star);
    if not bUnlock and bTips == true then
        local str = self:getGeniusLvUnlockLabel(config.talent_lvl);
        OperateAlert.show(LanUtil.formateStr(126091, str));
    end   
    return bUnlock;
end

function Lua_HeroGeniusData:checkGeniusLvIsUnlock(level, curStar)
	local list = self:getUnlockCond(level);
	if list then
		return curStar >= DataCenter.heroUtil:getHeroAwakenStarByQuality(list[1], list[2]);
	end
	return true;
end

function Lua_HeroGeniusData:getGeniusLvUnlockLabel(level)
	local list = self:getUnlockCond(level);
	if list then
		return DataCenter.heroUtil:getHeroQualityName(list[1], list[2]);
	end
	return nil;
end

function Lua_HeroGeniusData:checkGeniusLvCanActivate(cost_item, hero_material, heroUnId)
	local costId = DataCenter.packData:checkConsumeEnough(cost_item);
	if costId == nil then
		return DataCenter.heroMaterialData:checkEnoughByList(hero_material, heroUnId);
	end
	return false;
end

function Lua_HeroGeniusData:checkRedPoint(heroVo)
	if heroVo.lock_state then
		return false;
	end

	local arr = self:getHeroGeniusList(heroVo.id);
	if arr then
		local config;
		local skillId;
		local bUnlock;
		for i = 1, arr:getSize() do
			config = self:getGeniusConfig(arr[i]);
			skillId = self:getHeroGeniusSkill(heroVo.unique_id, config.talent_lvl);
			if skillId == 0 then
				bUnlock = self:checkGeniusLvIsUnlock(config.talent_lvl, heroVo.star);
				if bUnlock then
					return self:checkGeniusLvCanActivate(config.cost_item, config.hero_material, heroVo.unique_id)
				end
			end
		end
	end
	return false;
end