-- 主线前景控制器
checkRequire("Lua_MainLinePreViewConfig");
checkRequire("Lua_MainLineChapterConfigData");
Lua_MainLinePreViewController = class("Lua_MainLinePreViewController")
function Lua_MainLinePreViewController:ctor()
	self:superCall("ctor");
		
	self.viewLayerConst = 15;
	self.widgetsCountConst = 2;
	self.bModelBehavior = false;
	self._bOpenPreView = true;
	self._bShowPreView = false;
	self._chapterId = 0;
	self._lastTime = 0;

	self.bModelLoaded = false;
	self.bearModelId = 109100100;
	self._modelArr = Array();
	self._modelDic = {};
end

function Lua_MainLinePreViewController:dispose()
	self._chapterId = 0;
	self:clearWidgets();
	if self._viewRoot ~= nil then
		self._viewRoot:SetActive(false);
	end
end

-- 章节
function Lua_MainLinePreViewController.Get:chapterId()
	return self._chapterId;
end

-- 章节
function Lua_MainLinePreViewController.Set:chapterId(val)
	if self._chapterId ~= val then
		self._chapterId = val;
		self:updatePreView();
	end
end

function Lua_MainLinePreViewController.Get:bOpenPreView()
	return self._bOpenPreView;
end

function Lua_MainLinePreViewController.Set:bOpenPreView(val)
	if self._bOpenPreView ~= val then
		self._bOpenPreView = val;
		self:updatePreView();
	end
end

function Lua_MainLinePreViewController.Get:viewRoot()
	return self._viewRoot;
end

-- 更新
function Lua_MainLinePreViewController:frameScript()
	local t = getTime();
	local deltaTime = t - self._lastTime;
    self._lastTime = t;

    self:updateWidgets(deltaTime);
end

function Lua_MainLinePreViewController:updateWidgets(deltaTime)
	if self._scrollWidgetsArr == nil or self.bModelBehavior then
		return;
	end

	local widget = nil;
	local moveDistance = deltaTime * self._widgetMoveSpd;
	local posx = self._startPosX;
	posx = posx + moveDistance;
	if moveDistance < 0 then
		if posx <= -self._sceneWidth then
			posx = posx + self._sceneWidth;
			widget = self._scrollWidgetsArr[1];
			self._scrollWidgetsArr:remove(1);
			self._scrollWidgetsArr:add(widget);
		end

		self._startPosX = posx;
		for i = 1, self.widgetsCountConst do
			widget = self._scrollWidgetsArr[i];
			if widget ~= nil then
				if i > 1 then
					posx = posx + self._sceneWidth;
					widget:setLocation(posx, 0, 0);
				else
					widget:setLocation(posx, 0, 0);
				end
			end
		end
	else
		if posx >= self._sceneWidth then
			posx = posx - self._sceneWidth;
			widget = self._scrollWidgetsArr[1];
			self._scrollWidgetsArr:remove(1);
			self._scrollWidgetsArr:add(widget);
		end

		self._startPosX = posx;
		for i = 1, self.widgetsCountConst do
			widget = self._scrollWidgetsArr[i];
			if widget ~= nil then
				if i > 1 then
					posx = posx - self._sceneWidth;
					widget:setLocation(posx, 0, 0);
				else
					widget:setLocation(posx, 0, 0);
				end
			end
		end
	end
end

-- 初始化节点
function Lua_MainLinePreViewController:initPreViewRoot()
	if self._viewRoot ~= nil then return; end
	self._viewRoot = ContainerUtil.getContainer("ExtraRoot2", true);
	if self._viewRoot == nil then 
		self._viewRoot = UICreate.createGameObject(nil, 10000, 0);
		self._viewRoot.name = "ExtraRoot2";
	end
	if self._viewRoot ~= nil then
		self._widgetsRoot = UICreate.createGameObject(self._viewRoot);
		self._widgetsRoot.name = "widgetsRoot";
	end
end

-- 初始化相机
function Lua_MainLinePreViewController:initPreViewCamera()
	if self._viewRoot == nil then return; end
	if self._viewCam ~= nil then return; end
	self._viewCamRoot = UICreate.createGameObject(self._viewRoot);
	self._viewCamRoot.name = "viewCamRoot";
	self._viewCam = self._viewCamRoot:AddComponent(typeof(UnityEngine.Camera)); 
	self._viewCam.cullingMask = Lua_BitOperation.lShift(1, self.viewLayerConst);
	self._viewCam.nearClipPlane = 0.01;
	self._viewCam.farClipPlane = 100;
	self._viewCam.allowHDR = true;
	GlobalVar.addCamToStack(self._viewCam, 0);
end

function Lua_MainLinePreViewController:initScrollWidgets(resUrl, modelName)
	if strIsNullOrEmpty(resUrl) or strIsNullOrEmpty(modelName) then return; end
	if self._widgetsRoot == nil then return; end
	if self._scrollWidgetsArr == nil then
		self._scrollWidgetsArr = Array();
	end

	local widget = nil;
	local rx = (cfg ~= nil and cfg.groundRot ~= nil and #cfg.groundRot > 0) and cfg.groundRot[1] or 0;
	local ry = (cfg ~= nil and cfg.groundRot ~= nil and #cfg.groundRot > 1) and cfg.groundRot[2] or 0;
	local rz = (cfg ~= nil and cfg.groundRot ~= nil and #cfg.groundRot > 2) and cfg.groundRot[3] or 0;
	local len = self._scrollWidgetsArr:getSize();
	for i = 1, len do
		widget = self._scrollWidgetsArr[i];
		widget:setUrl(resUrl, modelName);
		widget:setRootRot(rx, ry, rz);
	end

	len = len + 1;
	for i = len, self.widgetsCountConst do
		widget = ModelEffComponent(self._widgetsRoot);
		widget.layer = self.viewLayerConst;
		widget:setUrl(resUrl, modelName);
		widget:setRootRot(rx, ry, rz);
		self._scrollWidgetsArr:add(widget);
	end
end

function Lua_MainLinePreViewController:clearWidgets()
	if self._scrollWidgetsArr == nil then return; end
	local len = self._scrollWidgetsArr:getSize();
	while (len > 0) do
		self._scrollWidgetsArr[len]:dispose();
		self._scrollWidgetsArr:remove(len);
		len = len - 1;
	end
end

function Lua_MainLinePreViewController:updatePreViewCamera(cfg)
	if cfg == nil then return; end
	if self._viewCamRoot == nil then return; end
	local px = (cfg ~= nil and cfg.camPos ~= nil and #cfg.camPos > 0) and cfg.camPos[1] or 0;
	local py = (cfg ~= nil and cfg.camPos ~= nil and #cfg.camPos > 1) and cfg.camPos[2] or 0;
	local pz = (cfg ~= nil and cfg.camPos ~= nil and #cfg.camPos > 2) and cfg.camPos[3] or 0;
	local rx = (cfg ~= nil and cfg.camRot ~= nil and #cfg.camRot > 0) and cfg.camRot[1] or 0;
	local ry = (cfg ~= nil and cfg.camRot ~= nil and #cfg.camRot > 1) and cfg.camRot[2] or 0;
	local rz = (cfg ~= nil and cfg.camRot ~= nil and #cfg.camRot > 2) and cfg.camRot[3] or 0;
	LuaUtil.setObjPos(self._viewCamRoot, px, py, pz);
	LuaUtil.setObjRot(self._viewCamRoot, rx, ry, rz);
	self:updateCamFieldView(cfg);
end

-- 适配
function Lua_MainLinePreViewController:updateCamFieldView(cfg)
	if self._viewCam == nil then return; end
	if cfg == nil then
		local chapterCfg = Lua_MainLineChapterConfigData.getConfig(self.chapterId);
		if chapterCfg == nil or chapterCfg.previewId == nil then
			return;
		end

		cfg = getConfig("mainline_preview", chapterCfg.previewId);
		if cfg == nil then
			return;
		end
	end

	if cfg == nil then return; end
	local fieldView = cfg.fieldView ~= nil and cfg.fieldView or 10;
	local adapterRate = CameraControl.Instance.adapterRate;
	print("updateCamFieldView", fieldView, adapterRate)
	if cfg.camType ~= 1 then -- 正交
		self._viewCam.orthographic = true;
		self._viewCam.orthographicSize = fieldView * adapterRate;
	else
		self._viewCam.orthographic = false;
		-- self._viewCam.fieldOfView = fieldView * adapterRate;
		self._viewCam.fieldOfView = fieldView;
	end
end

-- 更新前景显示
function Lua_MainLinePreViewController:updatePreView()
	if not self.bOpenPreView then
		self:clearWidgets();
		if self._viewRoot ~= nil then
			self._viewRoot:SetActive(false);
		end
		return;
	end

	local chapterCfg = Lua_MainLineChapterConfigData.getConfig(self.chapterId);
	if chapterCfg == nil or chapterCfg.previewId == nil then
		self:clearWidgets();
		if self._viewRoot ~= nil then
			self._viewRoot:SetActive(false);
		end
		return;
	end

	local cfg = getConfig("mainline_preview", chapterCfg.previewId);
	if cfg == nil then
		self:clearWidgets();
		if self._viewRoot ~= nil then
			self._viewRoot:SetActive(false);
		end
		return;
	end

	if self._viewRoot ~= nil then
		self._viewRoot:SetActive(true);
	end

	self._startPosX = 0;
	self._sceneWidth = cfg.width;
	self._widgetMoveSpd = cfg.scrollSpd;
	self._lastTime = getTime();
	self:initPreViewRoot();
	self:initPreViewCamera();
	self:initScrollWidgets(cfg.resUrl, cfg.modelName);
	self:updatePreViewCamera(cfg);
	self:updateWidgets(0);

	-- 刷新显示
	if self._bShowPreView then
		self:showPreView(true);
	else
		self:hidePreView(true);
	end
end

-- 显示
function Lua_MainLinePreViewController:showPreView(bForce)
	if self._bShowPreView and not bForce then
		return;
	end

	self._bShowPreView = true;
	if self._viewRoot ~= nil then
		self._viewRoot:SetActive(self.bOpenPreView);
	end

	if self._widgetsRoot == nil then
		return;
	end

	if self.bOpenPreView then
		self:showModel();
	end
end

-- 隐藏
function Lua_MainLinePreViewController:hidePreView(bForce)
	if not self._bShowPreView and not bForce then
		return;
	end

	self._bShowPreView = false;
	if self._viewRoot ~= nil then
		self._viewRoot:SetActive(false);
	end
end

function Lua_MainLinePreViewController:showModel()
	if self.bModelBehavior then return; end

	self.bModelLoaded = false;
	self.modelLoadedNum = 0;
	self.loadModelTotal = 0;

	local config = Lua_MainLineChapterConfigData.getConfig(self.chapterId);
	local modelId;
	local model;
	local posX;
	local posY;
	local posZ;
	local dir;
	local scale;
	local len = self._modelArr:getSize();
	config = getConfig("mainline_preview", config.previewId);
	if config and config.model_list then
		local dataLen = #config.model_list;
		len = len - dataLen;

		while (len > 0) do
			modelId = self._modelArr:remove();
			model = self:getModelById(modelId);
			if model then
				model:dispose();
				self._modelDic[modelId] = nil;
			end
			len = len - 1;
		end

		self.loadModelTotal = dataLen;
		local modelCfg;
		for i = 1, dataLen do
			modelId = config.model_list[i];
			modelCfg = Lua_ModelConfigData.getConfig(modelId);
			if self._modelArr[i] == nil then
				self._modelArr:add(modelId);
				model = ModelEffComponent(self._widgetsRoot);
				model.layer = self.viewLayerConst;
				model:setLuaCallBack(self, self.onModelLoaded, nil);
				model:setUrl(modelCfg.modelUrl, modelCfg.modelName);
				self._modelDic[modelId] = model;
			elseif self._modelArr[i] ~= modelId then
				model = self:getModelById(self._modelArr[i]);
				model:setEnable(true);
				model:setLuaCallBack(self, self.onModelLoaded, nil);
				model:setUrl(modelCfg.modelUrl, modelCfg.modelName);
				self._modelArr[i] = modelId;
			else
				model = self:getModelById(modelId);
				model:setEnable(true);
				self:onModelLoaded();
			end
			posX = (config.model_x and #config.model_x >= i) and config.model_x[i] or 0;
			posY = 0;
			posZ = (config.model_z and #config.model_z >= i) and config.model_z[i] or 0;
			dir = (config.model_dir and #config.model_dir >= i) and config.model_dir[i] or 0;
			scale = (config.model_scale and #config.model_scale >= i) and config.model_scale[i] or 1;
			model:setLocation(posX, posY, posZ);
			model:setRot(dir);
			model:setScale(scale);
			model:setAction(1);
		end
	else
		for i = 1, len do
			modelId = self._modelArr[i];
			model = self:getModelById(modelId);
			if model ~= nil then
				model:setEnable(false);
			end
		end
	end

	self:setBearAction(false);
end

function Lua_MainLinePreViewController:onModelLoaded()
	self.modelLoadedNum = self.modelLoadedNum + 1;
	if self.modelLoadedNum >= self.loadModelTotal then
		self.bModelLoaded = true;
	end
	-- print("onModelLoaded", self.modelLoadedNum, self.loadModelTotal, self.bModelLoaded)
end

function Lua_MainLinePreViewController:getModelById(modelId)
	if self._modelDic == nil or modelId == nil then
		return nil;
	end
	return self._modelDic[modelId];
end

function Lua_MainLinePreViewController:setBearAction(bChcekEff)
	local hookTime = DataCenter.hookData.hookTime;
	local action, effect = DataCenter.mainLineUtil:getHookEffectByTime(hookTime);
	self:updateBearModel(action, effect, bChcekEff);
end

function Lua_MainLinePreViewController:updateBearModel(action, effect, bChcekEff)
	if self._widgetsRoot == nil or not self.bOpenPreView or self.bModelBehavior then
		return;
	end

	local bearModel = self:getModelById(self.bearModelId);
	if bearModel then
		bearModel:setAction(action);
		if bChcekEff ~= false then
			bearModel:cleanHangerEff();
			bearModel:addHangerEff(effect);
		end
	end
end

function Lua_MainLinePreViewController:setModelBehavior(action, actName, normalizedTime, bubbleId)
	if not self.bModelLoaded or action == nil then
		return;
	end
	-- print("setModelBehavior", action, bubbleId, getTraceback())

	self.bModelBehavior = true;
	if bubbleId ~= nil and bubbleId > 0 then
		self:setModelBubble(bubbleId);
	end

	local modelId;
	local model;
	for i = 1, self._modelArr:getSize() do
		modelId = self._modelArr[i];
		model = self:getModelById(modelId);
		if model ~= nil and model.bEnable then
			if action == 1 and modelId == self.bearModelId then
				self.bModelBehavior = false;
				self:setBearAction(true);
			else
				if not strIsNullOrEmpty(actName) and normalizedTime ~= nil then
					model:setAction(action, actName, normalizedTime);
				else
					model:setAction(action);
				end
			end
		end
	end
end

function Lua_MainLinePreViewController:setBubbleBehavior(bubbles)
	if not self.bModelLoaded or bubbles == nil then
		return 0;
	end

	local arr = Array();
	local config;
	for i = 1, #bubbles do
		config = getConfig("mainline_bubble", bubbles[i]);
		if config.chapter == nil or config.chapter == 0 or config.chapter == self.chapterId then
			arr:add(bubbles[i]);
		end
	end
	
	local len = arr:getSize();
	local bubbleId = 0;
	if len > 1 then
		local idx = getRandomRange(len);
		bubbleId = arr[idx];
		while bubbleId == self._bubbleId do
			idx = getRandomRange(len);
			bubbleId = arr[idx];
		end
	elseif len == 1 then
		bubbleId = arr[len];
	end

	if bubbleId > 0 then
		self._bubbleId = bubbleId;

		local time = 0;
		while true do
			config = getConfig("mainline_bubble", bubbleId);
			if config and config.play_sec and config.play_sec > 0 then
				time = time + config.play_sec / 1000;
				if config.pause_sec and config.pause_sec > 0 then
					time = time + config.pause_sec / 1000;
				end
				bubbleId = bubbleId + 1;
			else
				break;
			end
		end

		self:setModelBubble(self._bubbleId);
		return time + 0.5;
	end
	return 0;
end

function Lua_MainLinePreViewController:setModelBubble(bubbleId)
	-- print("setModelBubble", bubbleId, getTraceback())
	Lua_MsgUtil.callFunc("MAINLINE_BUBBLE_SHOW", bubbleId);
end

function Lua_MainLinePreViewController:getBubbleModel(idx)
	local modelId;
	if idx == nil or idx == 0 then
		local len = self._modelArr:getSize();
		if len > 1 then
			idx = getRandomRange(len);
			modelId = self._modelArr[idx];
			while modelId == self._randomModelId do
				idx = getRandomRange(len);
				modelId = self._modelArr[idx];
			end
			self._randomModelId = modelId;
		elseif len == 1 then
			modelId = self._modelArr[len];
		end
	else
		modelId = self._modelArr[idx];
	end
	return modelId;
end