Lua_TaskInfoVo = class("Lua_TaskInfoVo")
function Lua_TaskInfoVo:ctor(id, config)
	self.id = id;
	self._config = config;
	self.conditionInfo = getConfig("total_task", self.conditionId);
end

function Lua_TaskInfoVo:dispose()
	self._progressMaxValue = nil;
	self._advanceMessage = nil;
	self._conditionId = nil;
	self.receiveStage = nil;
	if self.rewardInfo ~= nil then
		self.rewardInfo:dispose();
		self.rewardInfo:clear();
	end
end

---------------------------------------- 配置表 ----------------------------------------
function Lua_TaskInfoVo.Get:config()
	if self._config == nil then
		self._config = getConfig("task", self.id);
	end
	return self._config;
end

-- 开启等级
function Lua_TaskInfoVo.Get:level()
	return self.config.level;
end

-- 主类型
function Lua_TaskInfoVo.Get:mainType()
	return self.conditionInfo.task_type;
end

-- 描述
function Lua_TaskInfoVo.Get:desc()
	return self.conditionInfo.desc;
end

-- 参数个数
function Lua_TaskInfoVo.Get:count()
	return self.conditionInfo.count;
end

-- 排序
function Lua_TaskInfoVo.Get:sortIndex()
	if self._sortIndex == nil then
		local totalCount = DataCenter.taskUtil:getTaskListCountByType(self.mainType);
		self._sortIndex = totalCount - self.config.sortIndex;
	end
	return self._sortIndex;
end

-- 进度条的totalValue，如"x/totalValue"
function Lua_TaskInfoVo.Get:progressMaxValue()
	if self.eventId == 1005 then -- 主线关卡显示几关、排除挂机等
		self._progressMaxValue = getConfig("mainline_game_level", self.conditionInfo.progress, "number");
		self._progressMaxValue = self._progressMaxValue == nil and self.conditionInfo.progress or self._progressMaxValue;
	else
		self._progressMaxValue = self.conditionInfo.progress;
	end
	return self._progressMaxValue;
end

-- 条件数组值
function Lua_TaskInfoVo.Get:arrValue()
	return self.conditionInfo.condition;
end

-- 对应系统功能是否开启
function Lua_TaskInfoVo.Get:isOpen()
	return DataCenter.funcOpenData:isOpened(self.config.open_id, false);
end

-- 对应系统功能未开启时，显示信息
function Lua_TaskInfoVo.Get:advanceMessage()
	if self._advanceMessage == nil then
		self._advanceMessage = DataCenter.funcOpenUtil:getFuncOpenVoById(self.config.open_id).advanceMessage;
	end
	return self._advanceMessage;
end

-- 奖励
function Lua_TaskInfoVo.Get:reward()
	if self.rewardInfo == nil then
		self.rewardInfo = Array();

		local tvo;
		local obj;
		for i = 1, #self.conditionInfo.reward do
			obj = self.conditionInfo.reward[i];
			tvo = Lua_ToolVo.createToolVo(obj[1], obj[2]);
			self.rewardInfo:add(tvo);
		end
	end
	return self.rewardInfo;
end

-- 跳转界面或事件
function Lua_TaskInfoVo.Get:eventId()
	return self.conditionInfo.eventId;
end

-- 任务条件id
function Lua_TaskInfoVo.Get:conditionId()
	if self._conditionId == nil then
		self._conditionId = self.config.condition_id;
	end
	return self._conditionId;
end










---------------------------------------- 服务端数据 ----------------------------------------
-- 更新服务端数据
function Lua_TaskInfoVo:updateTaskVoInfo(progressValue, flag, conditionId)	
	self._curProgress = progressValue; -- 每日、每周单个任务的活跃进度
	self.receiveStage = flag; -- 领取状态：0未完成 1完成 2已领取
	self.conditionId = conditionId; -- 任务总表的id
end

function Lua_TaskInfoVo.Get:curProgress()
	return self._curProgress or 0;
end

-- 任务进阶（成就任务）
function Lua_TaskInfoVo.Set:conditionId(value)
	if self._conditionId == value then
		return;
	end
	self._conditionId = value;
	self.conditionInfo = getConfig("total_task", self.conditionId);
	self.rewardInfo = nil;
end

-- 每日、每周单个任务的活跃进度
function Lua_TaskInfoVo:getProgress()
	local showProgress = 0;
	if self.eventId == 1005 then -- 主线关卡显示几关、排除挂机等
		showProgress = getConfig("mainline_game_level", self.curProgress, "number");
		showProgress = showProgress or self.curProgress;
	else
		showProgress = self.curProgress;
	end
	return showProgress or 0;
end

-- 进度条完成程度的文本
function Lua_TaskInfoVo:getProgressText()
	local langId = self.mainType == Lua_TaskData.achievementTaskTask and 202811 or 202812;
	return LanUtil.formateStr(langId, self:getProgress(), self.progressMaxValue);
end

-- 日常、周常、成绩任务奖励的领取状态：0未完成 1完成 2已领取
function Lua_TaskInfoVo:getReceiveStage()
	if not self.isOpen then
		return 0;
	end
	return self.receiveStage or 0;
end

-- 权重 完成 > 未完成 > 未解锁 > 已领取
function Lua_TaskInfoVo.Get:sortWeight()
	local weight;
	if not self.isOpen then
		 weight = 1000 + self.sortIndex;
	else
		weight = (self:getReceiveStage() + 1) % 3 * 10000 + self.sortIndex;
	end
	return weight;
end