Lua_TotemData = class("Lua_TotemData")
function Lua_TotemData:ctor()
	self:superCall("ctor");
	self.heroDic = {}; -- 英雄装备的图腾信息
	self.totemDic = {}; -- 图腾道具Vo
	self.totemList = Array(); -- 图腾道具唯一ID
	self.heroPartDic = {}; -- 英雄装备的图腾部位 道具唯一ID
	self.totemToHeroDic = {}; -- 图腾装备的英雄
	self.packPartDic = {}; -- 背包中每个部位的图腾
end

function Lua_TotemData:dispose()
	self.heroDic = {};
	self.totemDic = {};
	self.totemList:clear();
	if self.heroPartDic then
		for _, v in pairs(self.heroPartDic) do
			v:clear();
		end
	end
	self.totemToHeroDic = {};
	if self.packPartDic then
		for _, v in pairs(self.packPartDic) do
			v:clear();
		end
	end
end

-- 更新英雄装备的图腾数据
function Lua_TotemData:updateHeroTotemItem(heroUid, totemInfo, trialState)
	local uniqueId;
	local vo;
	local haveDic = {};
	local list = totemInfo ~= nil and totemInfo.totem_item or nil;
	if list then
		for i = 1, #list do
			uniqueId = tonumber(list[i].unique_id);
			vo = self.totemDic[uniqueId];
			if vo == nil then
				vo = Lua_ToolVo.createToolVo(list[i].id);
				self.totemDic[uniqueId] = vo;
				if not trialState then
					self.totemList:add(uniqueId);
					if self.heroPartDic[vo.part] == nil then
						self.heroPartDic[vo.part] = Array();
					end
					self.heroPartDic[vo.part]:add(uniqueId);
				end
			end
			vo:fromServer(list[i]);
			self.totemToHeroDic[uniqueId] = heroUid;

			haveDic[uniqueId] = 1;
		end
	end

	local info = self.heroDic[heroUid];
	if info and info.totem_item then
		for i = 1, #info.totem_item do
			uniqueId = tonumber(info.totem_item[i].unique_id);
			if haveDic[uniqueId] == nil and self:getHeroUidByTotem(uniqueId) == heroUid then
				self:removeSingleHeroTotemItem(uniqueId);
			end
		end	
	end	
	haveDic = nil;

	self.heroDic[heroUid] = totemInfo;
end

function Lua_TotemData:removeHeroTotemItem(heroUid)
	local info = self.heroDic[heroUid];
	if info and info.totem_item then
		local uniqueId;
		for i = 1, #info.totem_item do
			uniqueId = tonumber(info.totem_item[i].unique_id);
			self:removeSingleHeroTotemItem(uniqueId);
		end	
	end
	self.heroDic[heroUid] = nil;
end

function Lua_TotemData:removeSingleHeroTotemItem(itemUid)
	local vo = self:getTotemItemVo(itemUid);
	if vo and self.heroPartDic[vo.part] then
		self.heroPartDic[vo.part]:removeItem(itemUid);
	end
	self.totemDic[itemUid] = nil;
	self.totemList:removeItem(itemUid);
	self.totemToHeroDic[itemUid] = nil;
end

-- 更新背包中的图腾数据
function Lua_TotemData:updatePackTotemItem(vo, bRemove)
	if vo == nil then
		return;
	end
	if bRemove == true then
		if self.packPartDic[vo.part] then
			self.packPartDic[vo.part]:removeItem(vo.unique_id);
		end
	else
		if self.packPartDic[vo.part] == nil then
			self.packPartDic[vo.part] = Array();
		end
		self.packPartDic[vo.part]:add(vo.unique_id);
	end
end

function Lua_TotemData:getHeroTotemInfo(heroUid)
	return self.heroDic[tonumber(heroUid)];
end

function Lua_TotemData:getTotemItemVo(itemUid)
	return self.totemDic[tonumber(itemUid)];
end

function Lua_TotemData:getHeroUidByTotem(itemUid)
	return self.totemToHeroDic[tonumber(itemUid)] or 0;
end





------------------------------ 穿戴替换 ------------------------------
function Lua_TotemData:getHeroWearNum(heroUid, suitCfg)
	local num = 0;
	local info = heroUid ~= nil and self:getHeroTotemInfo(heroUid) or nil;
	if info and info.totem_item then
		local vo;
		for i = 1, #info.totem_item do
			vo = self:getTotemItemVo(info.totem_item[i].unique_id);
			if vo and vo.sub_type == suitCfg.totem_type and vo.star >= suitCfg.need_star then
				num = num + 1;
			end
		end
	end
	return num;
end

function Lua_TotemData:getTotemArrByPart(part, ignoreItem, bIgnoreHero)
	local arr = Array();

	-- 英雄穿戴的
	local data = self.heroPartDic[part];
	if data and bIgnoreHero ~= true then
		for i = 1, data:getSize() do
			if data[i] ~= ignoreItem then
				arr:add(data[i]);
			end
		end
	end

	-- 背包
	data = self.packPartDic[part];
	if data then
		for i = 1, data:getSize() do
			if data[i] ~= ignoreItem then
				arr:add(data[i]);
			end
		end
	end

	arr:sortf(self, self.onReplaceSort);
	return arr;
end

function Lua_TotemData:onReplaceSort(a, b)
	local _a = self:getHeroUidByTotem(a);
	local _b = self:getHeroUidByTotem(b);
	if _a == 0 and _b > 0 then
		return false;
	elseif (_a == 0 and _b == 0) or (_a > 0 and _b > 0) then
		_a = DataCenter.packData:getItemVoByUnId(a);
		if _a == nil then
			_a = self:getTotemItemVo(a);
		end
		_b = DataCenter.packData:getItemVoByUnId(b);
		if _b == nil then
			_b = self:getTotemItemVo(b);
		end
		if _a.star < _b.star then
			return false;
		elseif _a.star == _b.star then
			if _a.strengthenLv < _b.strengthenLv then
				return false;
			elseif _a.strengthenLv == _b.strengthenLv then
				if _a.sub_type > _b.sub_type then
					return false;
				elseif _a.sub_type == _b.sub_type then
					if _a.part > _b.part then
						return false;
					elseif _a.part == _b.part then
						if _a.score < _b.score then
							return false;
						end
					end
				end
			end
		end
	end
	return true;
end





------------------------------ 属性 ------------------------------
function Lua_TotemData:getMainAttrArr(data)
	local arr = Array();
	local lvCfg = DataCenter.totemUtil:getLevelConfig(data.id, data.strengthenLv);
	local vo;
	if data.totem_info == nil or data.totem_info.prop == nil then
		for k, v in pairs(lvCfg.prop) do
			vo = DataCenter.attributesUtil:createAttrVo(k);
			vo.value = v;
			arr:add(vo);
		end
	else
		local list = data.totem_info.prop;
		local tempCfg = DataCenter.totemUtil:getLevelConfig(data.id, 0);
		for i = 1, #list do
			vo = DataCenter.attributesUtil:createAttrVo(list[i].name);
			vo.value = list[i].value;
			if lvCfg.prop and lvCfg.prop[vo.id] then
				vo.nextValue = lvCfg.prop[vo.id] - tempCfg.prop[vo.id];
			end
			arr:add(vo);
		end
	end
	arr:sort("sort", false);
	return arr;
end

function Lua_TotemData:getRandomAttrArr(data)
	local list = data.totem_info ~= nil and data.totem_info.entry_prop or nil;
	if list and #list > 0 then
		local exDic = {};
		local exList = data.totem_info.entry_prop_ex;
		local name;
		if exList then
			for i = 1, #exList do
				name = exList[i].name;
				if exDic[name] == nil then
					exDic[name] = exList[i].value;
				else
					exDic[name] = exDic[name] + exList[i].value;
				end
			end
		end

		local arr = Array();
		local vo;
		for i = 1, #list do
			vo = DataCenter.attributesUtil:createAttrVo(list[i].name);
			if vo then
				vo.value = list[i].value;
				vo.nextValue = exDic[vo.id] or 0;
				arr:add(vo);
			end
		end
		arr:sort("sort", false);
		return arr;
	end
	return nil;
end





------------------------------ 强化 ------------------------------
function Lua_TotemData:getStrengthenMaterials(ignoreUId, star)
	local arr = Array();

	-- 图腾晶石
	local data = DataCenter.packData:getUnIdList(Lua_PackDefine.MATERIALS, Lua_PackDefine.TOTEM_MATERIAL);
	for i = 1, data:getSize() do
		arr:add(data[i]);
	end

	-- 图腾道具 6星的图腾不能当作强化材料
	star = star == 0 and 5 or star;
	data = DataCenter.packData:getItemArrByPackType(Lua_PackDefine.TYPE_TOTEM);
	local vo;
	for i = 1, data:getSize() do
		vo = DataCenter.packData:getItemVoByUnId(data[i]);
		if data[i] ~= ignoreUId and vo.star <= star then
			arr:add(data[i]);
		end
	end	

	arr:sortf(self, self.onStrengthenMaterialsSort);
	return arr;
end

function Lua_TotemData:onStrengthenMaterialsSort(a, b)
	a = DataCenter.packData:getItemVoByUnId(a);
	b = DataCenter.packData:getItemVoByUnId(b);
	if a.main_type ~= 3 and b.main_type == 3 then
		return false;
	elseif a.main_type ~= 3 and b.main_type ~= 3 then
		if a.totemSort < b.totemSort then
			return false;
		end
	else
		if a.id > b.id then
			return false;
		end		
	end
	return true;
end





------------------------------ 合成 ------------------------------
function Lua_TotemData:getSynthesisMaterials(star)
	local arr = Array();

	-- 背包
	local data = DataCenter.packData:getItemArrByPackType(Lua_PackDefine.TYPE_TOTEM);
	local vo;
	for i = 1, data:getSize() do
		vo = DataCenter.packData:getItemVoByUnId(data[i]);
		if vo.star == star then
			arr:add(data[i]);
		end
	end

	arr:sortf(self, self.onSynthesisMaterialsSort);
	return arr;
end

function Lua_TotemData:onSynthesisMaterialsSort(a, b)
	a = DataCenter.packData:getItemVoByUnId(a);
	b = DataCenter.packData:getItemVoByUnId(b);
	if a.strengthenLv > b.strengthenLv then
		return false;
	elseif a.strengthenLv == b.strengthenLv then
		if a.sub_type > b.sub_type then
			return false;
		elseif a.sub_type == b.sub_type then
			if a.score > b.score then
				return false;
			end
		end
	end
	return true;
end





------------------------------ 红点 ------------------------------
function Lua_TotemData:checkRedPoint(heroUid)
	local partDic = {};
	local info = self.heroDic[heroUid];
	if info and info.totem_item then
		for i = 1, #info.totem_item do
			uniqueId = tonumber(info.totem_item[i].unique_id);
			vo = self:getTotemItemVo(uniqueId);
			if vo then
				partDic[vo.part] = vo;
			end
		end	
	end

	for i = 1, 6 do
		if self:checkWearReplaceRed(i, partDic[i]) then
			return true;
		end
	end
	partDic = nil;
	return false;
end

-- 穿戴/替换红点
function Lua_TotemData:checkWearReplaceRed(part, wearVo)
	local data = self.packPartDic[part];
	if data == nil then
		return false;
	end

	local len = data:getSize();
	if wearVo == nil and len > 0 then
		return true
	end

	local vo;
	for i = 1, len do
		vo = DataCenter.packData:getItemVoByUnId(data[i]);
		if vo.sub_type == wearVo.sub_type then
			if vo.star > wearVo.star then
				return true;
			elseif vo.star == wearVo.star then
				if vo.strengthenLv > wearVo.strengthenLv then
					return true;
				elseif vo.strengthenLv == wearVo.strengthenLv then
					if vo.score > wearVo.score then
						return true;
					end
				end
			end
		end
	end
	return false;
end