checkRequire("Lua_WarOrderRewardVo");
Lua_WarOrderData = class("Lua_WarOrderData");
Lua_WarOrderData.LightTower = 1; -- 光之塔战令
Lua_WarOrderData.Expedition = 2; -- 边境远征战令
Lua_WarOrderData.PeakDuel = 4; -- 巅峰对决战令
Lua_WarOrderData.EquipCopyWarMakes = 5; -- 寂灭岛战令
Lua_WarOrderData.KunXuHolyLandWarMakes = 6; -- 昆墟圣地战令
function Lua_WarOrderData:ctor()
	self:dispose();
end

function Lua_WarOrderData:dispose()
	self.advActiveList = {};
	self.redDotList = {};
end

function Lua_WarOrderData:fromServer50197021(msg)
	if msg == nil then
		return;
	end

	self.curType = msg.type; -- 类型
	self.curStep = msg.cur_step; -- 当前阶段
	self.curTarget = msg.cur_target; -- 当前目标值
	self.warOrderEndTick = msg.end_tick; -- 结束时间
	self.dayRewardState = msg.day_reward_state; -- 每日奖励状态 0不可领取 1可领取 2已领取
	self.curLvlId = msg.cur_level; -- 当前等级

	self.activeStepArr = Array();
	self.advActiveList[self.curType] = {}; -- 存储高级战令激活状态
	for k, v in ipairs(msg.step_state) do -- 是否激活高级战令（key：阶段，value：是否激活）
		if v.key <= self.curStep then
			self.activeStepArr:add(v);
		end
		self.advActiveList[self.curType][v.key] = v;
	end
	self.activeStepArr:sort("key", false);

	self.warOrderVoDic = {};
	self.warOrderIdDic = {};
	self.redDotList[self.curType] = {}; -- 存储红点数据

	local vo;
	local redDotState = false;
	local id = 0;
	for k, v in ipairs(msg.reward_state) do -- 奖励状态(key:索引id，value：普通奖励领取状态，other：高级奖励领取状态)
		vo = Lua_WarOrderRewardVo.new(v.key);
		if vo.config ~= nil then
			vo:fromServerInfo(v);
			self.warOrderVoDic[vo.id] = vo;

			if vo.show_cond ~= 0 then
				if self.warOrderIdDic[vo.step] == nil then
					self.warOrderIdDic[vo.step] = Array();
				end
				self.warOrderIdDic[vo.step]:add(vo.id);

				if vo.norState == 1 or vo.advState == 1 then
					redDotState = true;
					self.redDotList[self.curType][vo.step] = 1;
				end

				if id < vo.id then
					id = vo.id;
				end
			end
		end
	end
	self.maxLvlId = id;
	self:updateRedDotShow(redDotState);
	Lua_MsgUtil.callFunc("WAR_ORDER_INFO_UPDATE");
	DataCenter.equipCopyData:checkSingleRedDotState(); -- 寂灭岛战令红点
end

-- 更新红点显示
function Lua_WarOrderData:updateRedDotShow(state)
	if self.curType == Lua_WarOrderData.LightTower then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WarOrder_LightTower, state);
	elseif self.curType == Lua_WarOrderData.Expedition then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WarOrder_Expedition, state);
	elseif self.curType == Lua_WarOrderData.PeakDuel then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WarOrder_PeakDuel, state);
	elseif self.curType == Lua_WarOrderData.EquipCopyWarMakes then
		self.equipcopywarHasReceive = (state and DataCenter.funcOpenData:isOpened(10222)); -- 寂灭岛功能使用
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WarOrder_EquipCopy, self.equipcopywarHasReceive);
	elseif self.curType == Lua_WarOrderData.KunXuHolyLandWarMakes then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WarOrder_KunXuHolyLand, state);
		if DataCenter.funcOpenData:isOpened(10245) then
			DataCenter.redPointData:updateRedPointState(Lua_RedPointList.KunXuHolyLand_War, state); -- 昆墟功能那边也需红点
		end
	end
end

-- 更新阶段红点显示
function Lua_WarOrderData:getCurRedDotShow(step)
	if self.redDotList[self.curType] ~= nil then
		return self.redDotList[self.curType][step] ~= nil;
	end
	return false;
end

-- 获取战令Vo
function Lua_WarOrderData:getWarOrderVo(id)
	return self.warOrderVoDic[id];
end

-- 获取当前战令Vo
function Lua_WarOrderData.Get:getCurWarOrderVo()
	return self.warOrderVoDic[self.curLvlId];
end

-- 获取最大战令Vo
function Lua_WarOrderData.Get:getMaxWarOrderVo()
	return self.warOrderVoDic[self.maxLvlId];
end

-- 获取战令Id列表
function Lua_WarOrderData:getWarOrderIdArr(step, isSort)
	local arr = self.warOrderIdDic[step];
	if arr ~= nil and isSort then
		local a;
		local b;
		local len = arr:getSize();
		for i = 1, len do
			for j = i + 1, len do
				a = self:getWarOrderVo(arr[i]).sort;
				b = self:getWarOrderVo(arr[j]).sort;
				if a > b then
					local temp = arr[i];
					arr[i] = arr[j];
					arr[j] = temp;
				end
			end
		end
	end
	return arr;
end

-- 获取战令Vo列表
function Lua_WarOrderData:getWarOrderVoArr(step, isSort)
	local idArr = self:getWarOrderIdArr(step, isSort);
	local arr = Array();
	local vo;
	for i = 1, idArr:getSize() do
		vo = self:getWarOrderVo(idArr[i]);
		arr:add(vo);
	end
	return arr;
end

-- 获取默认阶段
function Lua_WarOrderData.Get:getDefaultStep()
	local step = self.curStep;
	for i = 1, self.activeStepArr:getSize() do
		if self:getCurRedDotShow(self.activeStepArr[i].key) then
			step = self.activeStepArr[i].key;
			break;
		end
	end
	return step;
end

-- 获取奖励的下标
function Lua_WarOrderData:getRewardIdx(step)
	local index = 1;
	local arr = self:getWarOrderVoArr(step, false);
	if arr ~= nil then
		local rewardState = self:getCurRedDotShow(step);
		if rewardState then
			for i = 1, arr:getSize() do
				if arr[i].norState == 1 or arr[i].advState == 1 then
					index = i;
					break;
				end
			end
		else
			local advActState = self:curAdvActState(step);
			for i = 1, arr:getSize() do
				if arr[i].norState == 0 or (advActState and arr[i].advState == 0) then
					index = i - 1;
					break;
				end
			end
		end
	end

	if index < 1 then index = 1 end
	return index;
end

-- 是否领完奖励
function Lua_WarOrderData:isExitReward(step)
	local arr = self:getWarOrderVoArr(step, false);
	if arr ~= nil then
		local vo;
		local advActive = self:curAdvActState(step);
		for i = 1, arr:getSize() do
			vo = arr[i];
			if vo.norState ~= 2 or (advActive and vo.advState ~= 2) then
				return true;
			end
		end
	end
	return false;
end

-- 累计奖励列表
function Lua_WarOrderData:getCumsumRewardArr(step)
	local arr = self:getWarOrderVoArr(step, false);
	local dic = {};
	for i = 1, arr:getSize() do
		for k, v in pairs(arr[i].adv_reward) do
			if dic[v[1]] == nil then
				dic[v[1]] = 0;
			end
			dic[v[1]] = dic[v[1]] + v[2];
		end
	end

	arr = Array();
	for k,v in pairs(dic) do
		arr:add({k, v});
	end
	return arr;
end

-- 获得高级奖励状态
function Lua_WarOrderData:advActiveState(type, step)
	local list = self.advActiveList[type];
	if list ~= nil then
		return list[step].value == 1;
	end
	return false;
end

-- 获取当前高级奖励状态
function Lua_WarOrderData:curAdvActState(step)
	return self:advActiveState(self.curType, step);
end

-- 获取高级战令满足领取却未解锁的
function Lua_WarOrderData:getAdvRewardIdx(step)
	local index = 0;
	local arr = self:getWarOrderVoArr(step, false);
	if arr ~= nil then
		for i = 1, arr:getSize() do
			if arr[i].advState == 0 then
				index = i;
				break;
			end
		end
	end
	return index;
end

-- 是否达到最高等级
function Lua_WarOrderData.Get:isMaxLevel()
	return self.maxLvlId == self.curLvlId;
end


---------------------------------------- 外部方法 ----------------------------------------

-- 提供寂灭岛外部使用，寂灭岛战令的红点是否显示
function Lua_WarOrderData:getEquipCopyWarMarkRedPoint()
	return self.equipcopywarHasReceive;
end
